package energycrops;

import java.util.ArrayList;
import java.util.Random;

import repast.simphony.context.Context;
import repast.simphony.context.space.continuous.ContinuousSpaceFactoryFinder;
import repast.simphony.context.space.grid.GridFactoryFinder;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.parameter.Parameters;
import repast.simphony.space.continuous.RandomCartesianAdder;
import repast.simphony.space.grid.GridBuilderParameters;
import repast.simphony.space.grid.RandomGridAdder;
import repast.simphony.util.ContextUtils;
import repast.simphony.valueLayer.GridValueLayer;
import repast.simphony.batch.BatchScenarioLoader;


public class PPContextCreator implements ContextBuilder<ProducerAgent> {
	
	
	public Context build(Context<ProducerAgent> context) {
		
		//System.out.println("first");
		
		ArrayList farmerList = new ArrayList();
		ArrayList initialfarmerList = new ArrayList();
		ArrayList landList = new ArrayList();
		ArrayList initiallandList = new ArrayList();

		Parameters p = RunEnvironment.getInstance().getParameters();
		int xdim = (Integer) p.getValue("worldxsize"); // The x dimension of the
		// physical space
		int ydim = (Integer) p.getValue("worldysize"); // The y dimension of the
		// physical space

		
		
		
		int start_run_number = ProducerAgent.getNumberOfRuns();
		
		
		
		Variables.setinitial_run_number(start_run_number);
		
		//System.out.println("run number" + Variables.getinitial_run_number());
		
		// Create a new 2D grid to model the discrete patches of grass. The
		// inputs to the
		// GridFactory include the grid name, the context in which to place the
		// grid,
		// and the grid parameters. Grid parameters include the border
		// specification,
		// random adder for populating the grid with agents, boolean for
		// multiple occupancy,
		// and the dimensions of the grid.

		GridFactoryFinder
				.createGridFactory(null)
				.createGrid(
						"Simple Grid",
						context,
						new GridBuilderParameters<ProducerAgent>(
								new repast.simphony.space.grid.WrapAroundBorders(),
								new RandomGridAdder<ProducerAgent>(), true,
								xdim, ydim));

		// Create a new 2D continuous space to model the physical space on which
		// the sheep
		// and wolves will move. The inputs to the Space Factory include the
		// space name,
		// the context in which to place the space, border specification,
		// random adder for populating the grid with agents,
		// and the dimensions of the grid.

		ContinuousSpaceFactoryFinder
				.createContinuousSpaceFactory(null)
				.createContinuousSpace(
						"Continuous Space",
						context,
						new RandomCartesianAdder<ProducerAgent>(),
						new repast.simphony.space.continuous.WrapAroundBorders(),
						xdim, ydim, 1);

		// Create a new 2D value layer to store the state of the grass grid.
		// This is
		// only used for visualization since it's faster to draw the value layer
		// in 2D displays compared with rendering each grass patch as an agent.

		GridValueLayer vl = new GridValueLayer("Land Field", true,
				new repast.simphony.space.grid.WrapAroundBorders(), xdim, ydim);
		context.addValueLayer(vl);

		// The environment parameters contain the user-editable values that
		// appear in the GUI.
		// Get the parameters p and then specifically the initial numbers of farmers
		
	
	
	
		
		int numFarmers = (Integer) p.getValue("initialnumberoffarmers");
		Variables.init_land_needs(numFarmers);
		
		int numLands = (Integer) p.getValue("totallands");
		int[] land_occupy_identitor = new int[xdim * ydim];
		// Populate the root context with the initial agents

		Variables Variables = new Variables();
		
		Generals G = new Generals(Variables);
		Output OP = new Output(Variables);
		
		context.add(G);
		context.add(OP);
		
		// calculate waste land
		
			
			
		int wasteLand = xdim * ydim - numLands;
		int[] rand_waste_lands = randperm(xdim);
		for (int i = 0; i < wasteLand; i++) {
			int waste_land_id = numLands + i + 1;
			SAMA sama = new SAMA(context, Variables, G, rand_waste_lands[i], 0,
					waste_land_id);
			land_occupy_identitor[rand_waste_lands[i] * ydim] = 1;
			if (sama.getpatch_leased() == true) {
				sama.setOwner(9999);
				sama.setrenter(9999);
			} else {
				sama.setrenter(9999);
				sama.setOwner(9999);
			}

		}

		// Iterate over the number of farmers

		int[] farms_homestead = randperm(xdim * ydim);

		int homestead;

		for (int i = 0; i < numFarmers; i++) {
			int j = i;
			while (land_occupy_identitor[farms_homestead[j]] == 1) {
				j = j + 1;
			}
			homestead = farms_homestead[j];
			int y = homestead % ydim;
			int x = (homestead - y) / ydim;

			Producer prod = new Producer(context, Variables, G, x, y, i+1);
			farmerList.add(prod);
			initialfarmerList.add(prod);
			land_occupy_identitor[homestead] = 1;
		}
		Variables.setfarmerlist(farmerList);
	
	
		
		// Populate the patch grid with land
		// Iterate over the dimensions of the patch grid

		int[] randFarmers = randperm(numFarmers);
		for (int i = 0; i < numFarmers; i++) {
			Producer prod = (Producer) farmerList.get(randFarmers[i]);
			ArrayList farmerLandList = new ArrayList();
			ArrayList initialfarmerLandList = new ArrayList();
			int begin_patch_id = prod.getbegin_patch_ID();
			int plots_needed = prod.getplots_needed();
			int[] farmLands = randperm(plots_needed);
			if (farmLands.length > 0){
				
				SAMA sama = new SAMA(context, Variables, G, prod.getX(), prod
						.getY(), begin_patch_id + farmLands[0]); // create a new land
				farmerLandList.add(sama);
				initialfarmerLandList.add(sama);
				landList.add(sama);
				initiallandList.add(sama);
				sama.setcurrent_producer(prod.getfarmer_name());
				if (sama.getpatch_leased() == true) {
					sama.setOwner(9999);
					sama.setrenter(prod.getfarmer_name());
				} else {
					sama.setrenter(9999);
					sama.setOwner(prod.getfarmer_name());
				}
				

				if (plots_needed > 1) {
					for (int j = 1; j < plots_needed; j++) {
						int min_land = 0;
						double min_distance = Math.sqrt(xdim * xdim + ydim
								* ydim) + 1;
						for (int k = 0; k < xdim * ydim; k++) {
							if (land_occupy_identitor[k] == 0) {
								int yk = k % ydim;
								int xk = (k - yk) / ydim;
								double distance = Math.sqrt(Math.pow(xk
										- prod.getX(), 2)
										+ Math.pow(yk - prod.getY(), 2));
								if (distance < min_distance) {
									min_land = k;
									min_distance = distance;
								}
							}
						}
						int yland = min_land % ydim;
						int xland = (min_land - yland) / ydim;

						SAMA sama1 = new SAMA(context, Variables, G, xland,
								yland, begin_patch_id + farmLands[j]); // create
						
						land_occupy_identitor[min_land] = 1;
						farmerLandList.add(sama1);
						landList.add(sama1);
						sama1.setcurrent_producer(prod.getfarmer_name());
						if (sama1.getpatch_leased() == true) {
							sama1.setOwner(9999);
							sama1.setrenter(prod.getfarmer_name());
						} else {
							sama1.setrenter(9999);
							sama1.setOwner(prod.getfarmer_name());
						}
						
					}
				}

			}
			prod.setmylandList(farmerLandList);					

			Variables.setlandlist(landList);			
			
			Variables.setoriginalcontextsize(context.size());
		
			
			if(Variables.getrestartmodel() == true){
				System.out.println("inside pp context creator");
				double endAt = (Double) p.getValue("runLength");
				RunEnvironment.getInstance().endAt(endAt);
				
				
				return context;
				
			
				
			}
		}
		
		return context;
		
		
	}

	
	public static int[] randperm(int n) {

		// return a random permutation of size n
		// that is an array containing a random permutation of values 0, 1, ...,
		// n-1

		Random randg = new Random();

		int[] perm = new int[n];
		for (int i = 0; i < n; i++) {
			perm[i] = i;
		}
		for (int i = 0; i < n - 1; i++) {
			int j = randg.nextInt(n - i) + i;
			// sawp perm[i] and perm[j]
			int temp = perm[i];
			perm[i] = perm[j];
			perm[j] = temp;
		}
		return perm;
	}
	
	
	

}
	

