package runmodel;

import java.io.*;
import java.util.Iterator;

import org.jmock.core.constraint.IsInstanceOf;

import energycrops.Generals;
import energycrops.Output;
import energycrops.Producer;
import energycrops.ProducerAgent;
import repast.simphony.batch.BatchScenarioLoader;
import repast.simphony.context.Context;
import repast.simphony.engine.controller.Controller;
import repast.simphony.engine.controller.DefaultController;
import repast.simphony.engine.environment.AbstractRunner;
import repast.simphony.engine.environment.ControllerRegistry;
import repast.simphony.engine.environment.DefaultRunEnvironmentBuilder;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunEnvironmentBuilder;
import repast.simphony.engine.environment.RunState;
import repast.simphony.engine.schedule.ISchedule;
import repast.simphony.engine.schedule.Schedule;
import repast.simphony.parameter.SweeperProducer;
import simphony.util.messages.MessageCenter;

public class Runner extends AbstractRunner {

	private static MessageCenter msgCenter = MessageCenter
			.getMessageCenter(Runner.class);

	private RunEnvironmentBuilder runEnvironmentBuilder;
	protected Controller controller;
	protected boolean pause = false;
	protected Object monitor = new Object();
	protected SweeperProducer producer;
	private ISchedule schedule;
	private Context context;

	public Runner() {
		runEnvironmentBuilder = new DefaultRunEnvironmentBuilder(this, true);
		controller = new DefaultController(runEnvironmentBuilder);
		controller.setScheduleRunner(this);
	}

	public void load(File scenarioDir) throws Exception {
		if (scenarioDir.exists()) {
			BatchScenarioLoader loader = new BatchScenarioLoader(scenarioDir);
			ControllerRegistry registry = loader.load(runEnvironmentBuilder);
			controller.setControllerRegistry(registry);
			RunEnvironment.init(null, controller.getScheduleRunner(), loader
					.getParameters(), true);
		} else {
			msgCenter.error("Scenario not found", new IllegalArgumentException(
					"Invalid scenario " + scenarioDir.getAbsolutePath()));
			return;
		}

		controller.runParameterSetters(RunEnvironment.getInstance()
				.getParameters());
	}

	public void runInitialize() {
		controller.runInitialize(null);
		RunState rs = RunState.getInstance();
		schedule = rs.getScheduleRegistry().getModelSchedule(); // get the
		// schedule
		context = (Context) rs.getMasterContext(); // get the main context of
		// the model
	}

	public void cleanUpRun() {
		controller.runCleanup();
	}

	public void cleanUpBatch() {
		controller.batchCleanup();
	}

	// returns the tick count of the next scheduled item
	public double getNextScheduledTime() {
		return ((Schedule) RunEnvironment.getInstance().getCurrentSchedule())
				.peekNextAction().getNextTime();
	}

	// returns the number of model actions on the schedule
	public int getModelActionCount() {
		return schedule.getModelActionCount();
	}

	// returns the number of non-model actions on the schedule
	public int getActionCount() {
		return schedule.getActionCount();
	}

	// Step the schedule
	public void step() {
		if (this.getModelActionCount() == 0) {
			this.setFinishing(true);
		}
		System.err.print(this.getActionCount());
		Iterator it = this.getAgentIterator();
		Output output = null;
		while (it.hasNext()) {
			ProducerAgent agent = (ProducerAgent) it.next();
			if (agent instanceof Output) {
				output = (Output) agent;
			} else {
				agent.step1();
				agent.step2();
				agent.step3();
				agent.step4();
			}
		}
		if(output!=null) {
			output.step4();
		}
		// schedule.execute();
	}

	// stop the schedule
	public void stop() {
		if (schedule != null)
			schedule.executeEndActions();
	}

	public void setFinishing(boolean fin) {
		schedule.setFinishing(fin);
	}

	public void execute(RunState toExecuteOn) {
		// required AbstractRunner stub. We will control the
		// schedule directly.
	}

	public Iterator getAgentIterator() { // get the collection of the agents
		Iterator it = context.getObjects(ProducerAgent.class).iterator();
		return it;
	}

	public static void main(String[] args) {
		System.err.println(new Runner().getModelActionCount());
	}

	public int getNumberOfAgenst() {
		int res = 0;
		Iterator it = this.getAgentIterator();
		while (it.hasNext()) {
			Object obj = it.next();
			if (obj instanceof Producer)
				res++;
		}
		return res;
	}
}
